@@ -3,7 +3,9 @@
 """
 Test elements extra logic from svg xml lxml custom classes.
 """
+
 import os
+import sys
 from importlib import resources
 
 from lxml import etree
@@ -100,9 +102,7 @@ class InxMixin:
 
     def introspect_arg_parser(self, arg_parser):
         """Pull apart the arg parser to find out what we have in it"""
-        for (
-            action
-        ) in arg_parser._optionals._actions:  # pylint: disable=protected-access
+        for action in arg_parser._optionals._actions:  # pylint: disable=protected-access
             for opt in action.option_strings:
                 # Ignore params internal to inkscape (thus not in the inx)
                 if opt.startswith("--") and opt[2:] not in INTERNAL_ARGS:
@@ -154,13 +154,27 @@ def _load_inx_schemas():
         ".schema": etree.Schematron,  # "pre-ISO-Schematron"
     }
 
-    for name in resources.contents(__package__):
+    if sys.version_info > (3, 9):
+
+        def _contents(pkg):
+            return [path.name for path in resources.files(pkg).iterdir()]
+    else:
+        _contents = resources.contents
+
+    for name in _contents(__package__):
         _, ext = os.path.splitext(name)
         schema_class = _SCHEMA_CLASSES.get(ext)
         if schema_class is None:
             continue
 
-        with resources.open_binary(__package__, name) as fp:
+        if sys.version_info > (3, 9):
+
+            def _open_binary(pkg, res):
+                return resources.files(pkg).joinpath(res).open("rb")
+        else:
+            _open_binary = resources.open_binary
+
+        with _open_binary(__package__, name) as fp:
             schema_doc = etree.parse(fp)
         yield name, schema_class(schema_doc)
 
